-- FuBar_AlarmClockFu 0.2
-- Author: Anthony White
-- Purpose: A simple alarm clock for FuBar
-- See Changelog.txt for revision information

AlarmClockFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

local FMT_MIL = 0 -- 24 hour format
local FMT_NORM = 1 -- 12 hour format

AlarmClockFu.hour = 0
AlarmClockFu.minute = 0
AlarmClockFu.lastalarmmin = 0

-- Register the saved variable DB
AlarmClockFu:RegisterDB("AlarmClockFuDB")

-- Realm defaults for saved variables
AlarmClockFu:RegisterDefaults('profile', {
	offset = 0, -- the offset from server to local time
	hourfmt = FMT_NORM,
	alarmenable = true,
	alarmmin = 0,
	alarmhr = 0
})

-- Menu Request
AlarmClockFu.OnMenuRequest = {
	type = 'group',
	args = {
		offset = {
			type = 'range',
			name = "Offset",
			desc = "Time offset from server time to local time",
			get = "GetOffset",
			set = "SetOffset",
			min = -12,
			max = 12,
			step = 1
		},
		hourfmt = {
			type = 'group',
			desc = "Select a time format",
			name = "Hour Format",
			args = {
				normalfmt = {
					type = 'toggle',
					name = "12 Hour Format",
					desc = "Select a 12 hour time format",
					set = "SetNormalFormat",
					get = "IsNormalFormat"
				},
				militaryfmt = {
					type = 'toggle',
					name = "24 Hour Format",
					desc = "Select a 24 hour time format",
					set = "SetMilitaryFormat",
					get = "IsMilitaryFormat"
				}
			}
		},
		alarm = {
			type = 'group',
			name = "Alarm Settings",
			desc = "Change alarm settings",
			args = {
				alarmenable = {
					type = 'toggle',
					name = "Enable Alarm",
					desc = "Switch the alarm on or off",
					get = "IsAlarmEnabled",
					set = "SetAlarmEnabled"
				},
				alarmtime = {
					type = 'group',
					name = "Alarm Time",
					desc = "Set the alarm time",
					args = {
						alarmhr = {
							type = 'text',
							name = "Alarm Hour",
							desc = "Set the alarm hour",
							usage = "<hour>",
							get = "GetAlarmHour",
							set = "SetAlarmHour"
						},
						alarmmin = {
							type = 'text',
							name = "Alarm Minute",
							desc = "Set the alarm minute",
							usage = "<minute>",
							get = "GetAlarmMinute",
							set = "SetAlarmMinute"
						}
					}
				}
			}
		}
	}
}

-- Get / Set Methods
function AlarmClockFu:GetOffset()
	return self.db.profile.offset
end

function AlarmClockFu:SetOffset(v)
	self.db.profile.offset = v
end

function AlarmClockFu:IsMilitaryFormat()
	if self.db.profile.hourfmt == FMT_MIL then
		return true
	end
	
	return false
end

function AlarmClockFu:IsNormalFormat()
	if self.db.profile.hourfmt == FMT_NORM then
		return true
	end
	
	return false
end

function AlarmClockFu:SetMilitaryFormat()
	self.db.profile.hourfmt = FMT_MIL
end

function AlarmClockFu:SetNormalFormat()
	self.db.profile.hourfmt = FMT_NORM
end

function AlarmClockFu:GetAlarmHour()
	return self.db.profile.alarmhr
end

function AlarmClockFu:SetAlarmHour(hour)
	self.db.profile.alarmhr = tonumber(hour)
end

function AlarmClockFu:GetAlarmMinute()
	return self.db.profile.alarmmin
end

function AlarmClockFu:SetAlarmMinute(min)
	self.db.profile.alarmmin = tonumber(min)
end

function AlarmClockFu:IsAlarmEnabled()
	return self.db.profile.alarmenable
end

function AlarmClockFu:SetAlarmEnabled()
	self.db.profile.alarmenable = not self.db.profile.alarmenable
end

-- Formats the text displayed for the addon
function AlarmClockFu:FormatClockString()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour

	hour = hour + self.db.profile.offset
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- used to get the server time
function AlarmClockFu:FormatClockNoOffset()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- formats the alarm time
function AlarmClockFu:FormatAlarmString()
	local hour = self.db.profile.alarmhr
	local minute = self.db.profile.alarmmin

	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

function AlarmClockFu:GetNextMinute(curmin)
	if (curmin == 60) then
		return 0
	else
		return curmin + 1
	end
end

-- Called when the addon is initialized
function AlarmClockFu:OnInitialize()
	self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)
	self:RegisterEvent("AlarmClockFu_AlarmTrigger", self.AlarmClockTrigger, true)
end

-- Sets the displayed text
function AlarmClockFu:OnTextUpdate()
	self.hour, self.minute = GetGameTime()
	
	local timestr, _, _ = self:FormatClockString()
	
	self:SetText(timestr)
end

-- Make sure we update the text
function AlarmClockFu:OnUpdate()
	self:UpdateText()
	
	local oldformat = self.db.profile.hourfmt
	
	if (oldformat == FMT_NORM) then
		self.db.profile.hourfmt = FMT_MIL
	end
	
	local _, hour, min = self:FormatClockString()
	
	self.db.profile.hourfmt = oldformat
	
	if (self:GetNextMinute(self.lastalarmmin) == self.minute) then
		self:RegisterEvent("AlarmClockFu_AlarmTrigger", self.AlarmClockTrigger)
	end
	
	if (self.db.profile.alarmenable == true) then
		if (hour == self.db.profile.alarmhr and min == self.db.profile.alarmmin) then
			self:TriggerEvent("AlarmClockFu_AlarmTrigger")
			self.lastalarmmin = self.minute
			if (self:IsEventRegistered("AlarmClockFu_AlarmTrigger")) then
				self:UnregisterEvent("AlarmClockFu_AlarmTrigger")
			end
		end
	end
end

-- Event handler for AlarmClockFu_AlarmTrigger event
function AlarmClockFu:AlarmClockTrigger()
	_ERRORMESSAGE("Alarm Activated!")
	self:Print("Alarm Activated!")
end

local tablet = AceLibrary("Tablet-2.0")
function AlarmClockFu:OnTooltipUpdate()
	local timestr, _, _ = self:FormatClockString()
	local timestr_no_offset, _, _ = self:FormatClockNoOffset()
	local alarmtime, _, _ = self:FormatAlarmString()

	cat = tablet:AddCategory(
		'text', "Time",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', 1,
		'child_textG2', 1,
		'child_textB2', 1
	)
	
	cat:AddLine(
		'text', "Local Time",
		'text2', timestr
	)
	
	cat:AddLine(
		'text', "Server Time",
		'text2', timestr_no_offset
	)
	
	local r, g, b
	
	if (self.db.profile.alarmenable == false) then
		r = 1
		g = 0
		b = 0
	else
		r = 0
		g = 1
		b = 0
	end
	
	cat = tablet:AddCategory(
		'text', "Alarm",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled
	
	if self.db.profile.alarmenable == false then
		enabled = "No"
	else
		enabled = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled
	)
	
	if self.db.profile.alarmenable == true then
		cat:AddLine(
			'text', "Alarm Time",
			'text2', alarmtime
		)
	end
end